@@ -14,7 +14,7 @@
 #'   \item uses a custom highlight scheme
 #'  }
 #'
-#' See the \href{https://rmarkdown.rstudio.com/package_vignette_format.html}{online
+#' See the \href{https://bookdown.org/yihui/rmarkdown/r-package-vignette.html}{online
 #' documentation} for additional details on using the \code{html_vignette} format.
 #' @inheritParams html_document
 #' @param ... Additional arguments passed to \code{\link{html_document}}. Please
@@ -51,6 +51,11 @@ html_vignette <- function(fig_width = 3,
     }
   }
 
+  pre_processor <- function(metadata, input_file, runtime, knit_meta,
+                            files_dir, output_dir) {
+    vignette_pre_processor(input_file, metadata)
+  }
+
   output_format(
     knitr = NULL,
     pandoc = NULL,
@@ -58,6 +63,7 @@ html_vignette <- function(fig_width = 3,
     pre_knit = pre_knit,
     keep_md = keep_md,
     clean_supporting = self_contained,
+    pre_processor = pre_processor,
     base_format = html_document(fig_width = fig_width,
                                 fig_height = fig_height,
                                 dev = dev,
@@ -69,3 +75,24 @@ html_vignette <- function(fig_width = 3,
                                 ...)
   )
 }
+
+vignette_pre_processor <- function(input_file, metadata = yaml_front_matter(input_file)) {
+  if (getRversion() < 3.6)
+    return()
+  # TODO: use xfun::is_R_CMD_check() in the future (don't bump xfun >= 0.19 yet)
+  if (!getOption(o <- 'rmarkdown.html_vignette.check_title', is.na(Sys.getenv('_R_CHECK_PACKAGE_NAME_', NA))))
+    return()
+  title1 <- metadata[['title']]
+  title2 <- tools::vignetteInfo(input_file)[['title']]
+  # rmarkdown assumes UTF-8 only - tools::vignetteInfo uses readLines with
+  # default OS encoding so issue on Windows for example
+  # https://github.com/rstudio/rmarkdown/issues/1978
+  Encoding(title2) <- "UTF-8"
+  if (!identical(title1, title2)) warning(
+    'The vignette title specified in \\VignetteIndexEntry{} is different from ',
+    'the title in the YAML metadata. The former is "', title2, '", and the ',
+    'latter is "', title1, '". If that is intentional, you may set options(', o,
+    ' = FALSE) to suppress this check.', call. = FALSE
+  )
+  NULL
+}
