@@ -3,7 +3,7 @@
 #' Convert documents to and from various formats using the pandoc utility.
 #'
 #' Supported input and output formats are described in the
-#' \href{http://johnmacfarlane.net/pandoc/README.html}{pandoc user guide}.
+#' \href{https://pandoc.org/MANUAL.html}{pandoc user guide}.
 #'
 #' The system path as well as the version of pandoc shipped with RStudio (if
 #' running under RStudio) are scanned for pandoc and the highest version
@@ -28,13 +28,13 @@
 #'
 #' # convert markdown to various formats
 #' pandoc_convert("input.md", to = "html")
-#' pandoc_convert("input.md", to = "pdf")
+#' pandoc_convert("input.md", to = "latex")
 #'
 #' # process citations
 #' pandoc_convert("input.md", to = "html", citeproc = TRUE)
 #'
 #' # add some pandoc options
-#' pandoc_convert("input.md", to="pdf", options = c("--listings"))
+#' pandoc_convert("input.md", to = "latex", options = c("--listings"))
 #' }
 #' @export
 pandoc_convert <- function(input,
@@ -49,6 +49,9 @@ pandoc_convert <- function(input,
   # ensure we've scanned for pandoc
   find_pandoc()
 
+  # evaluate path arguments before changing working directory
+  force(output)
+
   # execute in specified working directory
   if (is.null(wd)) {
     wd <- base_dir(input)
@@ -61,6 +64,7 @@ pandoc_convert <- function(input,
   args <- c(input)
   if (!is.null(to)) {
     if (to == 'html') to <- 'html4'
+    if (to == 'pdf') to <- 'latex'
     args <- c(args, "--to", to)
   }
   if (!is.null(from))
@@ -71,18 +75,17 @@ pandoc_convert <- function(input,
     args <- c(args, "--output", output)
 
   # set pandoc stack size
-  stack_size <- getOption("pandoc.stack.size", default = "512m")
-  args <- c(c("+RTS", paste0("-K", stack_size), "-RTS"), args)
+  args <- prepend_pandoc_stack_size(args)
 
   # additional command line options
   args <- c(args, options)
 
   # citeproc filter if requested
   if (citeproc) {
-    args <- c(args, "--filter", pandoc_citeproc())
-    # --natbib/--biblatex conflicts with '--filter pandoc-citeproc'
-    i <- stats::na.omit(match(c("--natbib", "--biblatex"), options))
-    if (length(i)) options <- options[-i]
+    # --natbib/--biblatex conflicts with pandoc own citation processing,
+    # '--filter pandoc-citeproc' or '--citeproc'
+    args <- c(args[!args %in% c("--natbib", "--biblatex")],
+              pandoc_citeproc_args())
   }
 
   # build the conversion command
@@ -123,14 +126,26 @@ pandoc_citeproc_convert <- function(file, type = c("list", "json", "yaml")) {
   # resolve type
   type <- match.arg(type)
 
+  if (pandoc_available("2.11")) {
+    bin <- pandoc()
+    to <- switch(type,
+                 list = "csljson",
+                 json = "csljson",
+                 yaml = "markdown"
+    )
+    args <- c(file, "-s", "-t", to)
+  } else {
+    bin <- pandoc_citeproc()
+    conversion <- switch(type,
+                         list = "--bib2json",
+                         json = "--bib2json",
+                         yaml = "--bib2yaml"
+    )
+    args <- c(conversion, file)
+  }
+
   # build the conversion command
-  conversion <- switch(type,
-    list = "--bib2json",
-    json = "--bib2json",
-    yaml = "--bib2yaml"
-  )
-  args <- c(conversion, file)
-  command <- paste(quoted(pandoc_citeproc()), paste(quoted(args), collapse = " "))
+  command <- paste(quoted(bin), paste(quoted(args), collapse = " "))
 
   # run the conversion
   with_pandoc_safe_environment({
@@ -249,6 +264,14 @@ pandoc_variable_arg <- function(name,
   c("--variable", if (missing(value)) name else paste(name, "=", value, sep = ""))
 }
 
+#' @rdname pandoc_args
+#' @export
+pandoc_metadata_arg <- function(name,
+                                value) {
+
+  c("--metadata", if (missing(value)) name else paste(name, "=", value, sep = ""))
+}
+
 
 #' @rdname pandoc_args
 #' @export
@@ -323,6 +346,20 @@ pandoc_toc_args <- function(toc,
   args
 }
 
+#' @section About Pandoc citeproc:
+#' For Pandoc version before 2.11, a pandoc filter \samp{pandoc-citeproc} is
+#' used. Since Pandoc 2.11, the feature is built-in and activated using
+#' \samp{--citeproc} flag. \samp{pandoc_citeproc_arg} will return the correct
+#' switches depending on the Pandoc version in use.
+#' @rdname pandoc_args
+#' @export
+pandoc_citeproc_args <- function() {
+  if (pandoc_available("2.11"))
+    "--citeproc"
+  else
+    c("--filter", pandoc_citeproc())
+}
+
 
 #' Transform path for passing to pandoc
 #'
@@ -544,15 +581,45 @@ is_highlightjs <- function(highlight) {
   !is.null(highlight) && (highlight %in% c("default", "textmate"))
 }
 
-# Scan for a copy of pandoc and set the internal cache if it's found.
-find_pandoc <- function(cache = TRUE) {
+#' Find the \command{pandoc} executable
+#'
+#' Searches for the \command{pandoc} executable in a few places and use the
+#' highest version found, unless a specific version is requested.
+#' @param cache Whether to search for \command{pandoc} again if a Pandoc
+#'   directory containing the \command{pandoc} executable of the expected
+#'   version (if provided) has been found previously. Search again if
+#'   \code{cache = FALSE}.
+#' @param dir A character vector of potential directory paths under which
+#'   \command{pandoc} may be found. If not provided, this function searches for
+#'   \command{pandoc} from the environment variable \var{RSTUDIO_PANDOC} (the
+#'   RStudio IDE will set this variable to the directory of Pandoc bundled with
+#'   the IDE), the environment variable \var{PATH}, and the directory
+#'   \file{~/opt/pandoc/}.
+#' @param version The version of Pandoc to look for (e.g., \code{"2.9.2.1"}). If
+#'   not provided, this function searches for the highest version under the
+#'   potential directories.
+#' @note Usually you do not need to install Pandoc if you use the RStudio IDE,
+#'   because the IDE has bundled a version of Pandoc. If you have installed a
+#'   version of Pandoc by yourself and want to use this version instead, you may
+#'   use the \code{dir} argument of this function.
+#' @return A list containing the directory and version of Pandoc (if found).
+#' @export
+#' @examples rmarkdown::find_pandoc()
+#' rmarkdown::find_pandoc(dir = '~/Downloads/Pandoc')
+#' rmarkdown::find_pandoc(version = '2.7.3')
+find_pandoc <- function(cache = TRUE, dir = NULL, version = NULL) {
 
-  if (!is.null(.pandoc$dir) && cache) return(invisible(as.list(.pandoc)))
+  if (!cache) set_pandoc_info(NULL)  # clear previously found pandoc path
+  if (!is.null(.pandoc$dir) && (is.null(version) || version == .pandoc$version))
+    return(as.list(.pandoc))
 
-  # define potential sources
-  sys_pandoc <- find_program("pandoc")
-  sources <- c(Sys.getenv("RSTUDIO_PANDOC"), if (nzchar(sys_pandoc)) dirname(sys_pandoc))
-  if (!is_windows()) sources <- c(sources, path.expand("~/opt/pandoc"))
+  # look up pandoc in potential sources unless user has supplied `dir`
+  sources <- if (length(dir) == 0) c(
+    Sys.getenv("RSTUDIO_PANDOC"),
+    dirname(find_program("pandoc")),
+    "~/opt/pandoc"
+  ) else dir
+  sources <- path.expand(sources)
 
   # determine the versions of the sources
   versions <- lapply(sources, function(src) {
@@ -564,19 +631,14 @@ find_pandoc <- function(cache = TRUE) {
   found_ver <- numeric_version("0")
   for (i in seq_along(sources)) {
     ver <- versions[[i]]
-    if (ver > found_ver) {
+    if ((!is.null(version) && ver == version) || (is.null(version) && ver > found_ver)) {
       found_ver <- ver
       found_src <- sources[[i]]
     }
   }
 
-  # did we find a version?
-  if (!is.null(found_src)) {
-    .pandoc$dir <- found_src
-    .pandoc$version <- found_ver
-  }
-
-  invisible(as.list(.pandoc))
+  set_pandoc_info(found_src, found_ver)
+  as.list(.pandoc)
 }
 
 # Get an S3 numeric_version for the pandoc utility at the specified path
@@ -592,6 +654,17 @@ get_pandoc_version <- function(pandoc_dir) {
   numeric_version(version)
 }
 
+set_pandoc_info <- function(dir, version = if (!is.null(dir)) get_pandoc_version(dir)) {
+  .pandoc$dir <- dir
+  .pandoc$version <- version
+}
+
+# prepend pandoc stack size arguments
+prepend_pandoc_stack_size <- function(args) {
+  stack_size <- getOption("pandoc.stack.size", default = "512m")
+  c(c("+RTS", paste0("-K", stack_size), "-RTS"), args)
+}
+
 # wrap a system call to pandoc so that LC_ALL is not set
 # see: https://github.com/rstudio/rmarkdown/issues/31
 # see: https://ghc.haskell.org/trac/ghc/ticket/7344
@@ -656,7 +729,6 @@ detect_generic_lang <- function() {
 
 # get the path to the pandoc binary
 pandoc <- function() {
-
   find_pandoc()
   file.path(.pandoc$dir, "pandoc")
 }
@@ -664,13 +736,20 @@ pandoc <- function() {
 
 # get the path to the pandoc-citeproc binary
 pandoc_citeproc <- function() {
-
   find_pandoc()
-  citeproc_path = file.path(.pandoc$dir, "pandoc-citeproc")
-  if (file.exists(citeproc_path))
-    citeproc_path
-  else
-    "pandoc-citeproc"
+  bin <- "pandoc-citeproc"
+  p <- file.path(.pandoc$dir, bin)
+  if (xfun::is_windows()) p <- xfun::with_ext(p, "exe")
+  if (file.exists(p)) p else bin
+}
+
+#' @rdname pandoc_args
+#' @param lua_files Character vector of file paths to Lua filter files. Paths
+#'   will be transformed by \code{\link{pandoc_path_arg}}.
+#' @export
+pandoc_lua_filter_args <- function(lua_files) {
+  # Lua filters was introduced in pandoc 2.0
+  if (pandoc2.0()) c(rbind("--lua-filter", pandoc_path_arg(lua_files)))
 }
 
 
@@ -716,7 +795,7 @@ pandoc2.0 <- function() {
 #' either the system path or shipped with RStudio.
 #'
 #' See the
-#' \href{http://pandoc.org/MANUAL.html}{pandoc manual}
+#' \href{https://pandoc.org/MANUAL.html}{pandoc manual}
 #' for pandoc commands.
 #'
 #' @export
