@@ -90,7 +90,7 @@ metadata <- list()
 #'
 #'   For more details on \code{knitr::spin} see the following documentation:
 #'
-#'   \url{http://yihui.org/knitr/demo/stitch/}
+#'   \url{https://yihui.org/knitr/demo/stitch/}
 #' @name compile_notebook
 NULL
 
@@ -151,7 +151,7 @@ NULL
 #' \code{\link{rmarkdown_format}} for details.
 #' @seealso
 #' \link[knitr:knit]{knit}, \link{output_format},
-#' \href{http://johnmacfarlane.net/pandoc}{pandoc}
+#' \url{https://pandoc.org}
 #' @inheritParams default_output_format
 #' @param input The input file to be rendered. This can be an R script (.R),
 #' an R Markdown document (.Rmd), or a plain markdown document.
@@ -163,6 +163,11 @@ NULL
 #' format object (e.g. \code{html_document()}). If using \code{NULL} then the
 #' output format is the first one defined in the YAML frontmatter in the input
 #' file (this defaults to HTML if no format is specified there).
+#' If you pass an output format object to \code{output_format}, the options
+#' specified in the YAML header or \code{_output.yml} will be ignored and you
+#' must explicitly set all the options you want when you construct the object.
+#' If you pass a string, the output format will use the output parameters in
+#' the YAML header or \code{_output.yml}.
 #' @param output_file The name of the output file. If using \code{NULL} then the
 #' output filename will be based on filename for the input file. If a filename
 #' is provided, a path to the output file can also be provided. Note that the
@@ -227,9 +232,16 @@ NULL
 #' # Render all formats defined in the file
 #' render("input.Rmd", "all")
 #'
-#' # Render a single format
+#' # Render a single format, using parameters for \code{html_document} from
+#' # the YAML header parameters.
 #' render("input.Rmd", "html_document")
 #'
+#' # Render a single format, ignoring parameters for \code{html_document} in
+#' # the YAML header. Any parameters not passed as arguments to
+#' # \code{html_document()} will be assigned to their default values, regardless
+#' # of anything in the YAML header
+#' render("input.Rmd", html_document(toc = TRUE, toc_depth = 2))
+#'
 #' # Render multiple formats
 #' render("input.Rmd", c("html_document", "pdf_document"))
 #' }
@@ -242,7 +254,7 @@ render <- function(input,
                    output_yaml = NULL,
                    intermediates_dir = NULL,
                    knit_root_dir = NULL,
-                   runtime =  c("auto", "static", "shiny", "shiny_prerendered"),
+                   runtime =  c("auto", "static", "shiny", "shinyrmd", "shiny_prerendered"),
                    clean = TRUE,
                    params = NULL,
                    knit_meta = NULL,
@@ -330,7 +342,7 @@ render <- function(input,
   }
 
   # check whether this document requires a knit
-  requires_knit <- tolower(tools::file_ext(input)) %in% c("r", "rmd", "rmarkdown")
+  requires_knit <- tolower(xfun::file_ext(input)) %in% c("r", "rmd", "rmarkdown")
 
   # remember the name of the original input document (we overwrite 'input' once
   # we've knitted)
@@ -357,14 +369,16 @@ render <- function(input,
     if (is.null(intermediates_dir)) {
       intermediates_dir <-
         dirname(normalize_path(input_no_shell_chars))
-      # never use the original input directory as the intermediate directory,
-      # otherwise external resources discovered will be deleted as intermediate
-      # files later (because they are copied to the "intermediate" dir)
-      if (same_path(intermediates_dir, dirname(original_input)))
-        intermediates_dir <- NULL
     }
   }
 
+  # never use the original input directory as the intermediate directory,
+  # otherwise external resources discovered will be deleted as intermediate
+  # files later (because they are copied to the "intermediate" dir)
+  if (!is.null(intermediates_dir) &&
+      same_path(intermediates_dir, dirname(original_input)))
+    intermediates_dir <- NULL
+
   # force evaluation of knitr root dir before we change directory context
   force(knit_root_dir)
 
@@ -383,10 +397,10 @@ render <- function(input,
   intermediates <- c(intermediates, utf8_input)
 
   # track whether this was straight markdown input (to prevent keep_md later)
-  md_input <- identical(tolower(tools::file_ext(input)), "md")
+  md_input <- identical(tolower(xfun::file_ext(input)), "md")
 
   # if this is an R script then spin it first
-  if (identical(tolower(tools::file_ext(input)), "r")) {
+  if (identical(tolower(xfun::file_ext(input)), "r")) {
     # make a copy of the file to spin
     spin_input <- intermediates_loc(file_with_meta_ext(input, "spin", "R"))
     file.copy(input, spin_input, overwrite = TRUE)
@@ -398,17 +412,17 @@ render <- function(input,
                             format = "Rmd")
     intermediates <- c(intermediates, spin_rmd)
     knit_input <- spin_rmd
-    # append default metadata (this will be ignored if there is user
-    # metadata elsewhere in the file)
-    metadata <- paste('\n',
-      '---\n',
-      'title: "', input, '"\n',
-      'author: "', Sys.info()[["user"]], '"\n',
-      'date: "', Sys.Date(), '"\n',
-      '---\n'
-    , sep = "")
-    input_lines <- read_utf8(knit_input)
-    write_utf8(c(input_lines, metadata), knit_input)
+    # append default metadata unless the field exists in YAML
+    meta1 <- yaml_front_matter(knit_input)
+    meta2 <- list(
+      title = input, author = Sys.info()[["user"]],
+      date = as.character(Sys.Date())
+    )
+    for (i in names(meta2)) if (!is.null(meta1[[i]])) meta2[[i]] <- NULL
+    if (length(meta2)) {
+      input_lines <- read_utf8(knit_input)
+      write_utf8(c(input_lines, '\n\n---', yaml::as.yaml(meta2), '---'), knit_input)
+    }
   }
 
   # read the input file
@@ -427,19 +441,19 @@ render <- function(input,
   shiny_prerendered_dependencies <- list()
   if (requires_knit && is_shiny_prerendered(front_matter$runtime)) {
 
-    # first validate that the user hasn't passed an already created output_format
-    if (is_output_format(output_format)) {
-      stop("You cannot pass a fully constructed output_format to render when ",
-           "using runtime: shiny_prerendered")
-    }
-
     # require shiny for the knit
     if (requireNamespace("shiny")) {
       if (!"package:shiny" %in% search())
         attachNamespace("shiny")
     }
     else
-      stop("The shiny package is required for 'shiny_prerendered' documents")
+      stop("The shiny package is required for shinyrmd documents")
+
+    # source global.R if it exists
+    global_r <- file.path.ci(".", "global.R")
+    if (file.exists(global_r)) {
+      source(global_r, local = envir)
+    }
 
     # force various output options
     output_options$self_contained <- FALSE
@@ -517,6 +531,20 @@ render <- function(input,
   context <- render_context()
   context$df_print <- resolve_df_print(output_format$df_print)
 
+  # make the front_matter available as 'metadata' within the knit environment
+  # (unless it is already defined there, in which case we emit a warning)
+  env <- environment(render)
+  metadata_this <- env$metadata
+  do.call("unlockBinding", list("metadata", env))
+  on.exit({
+    if (bindingIsLocked("metadata", env)) {
+      do.call("unlockBinding", list("metadata", env))
+    }
+    env$metadata <- metadata_this
+    lockBinding("metadata", env)
+  }, add = TRUE)
+  env$metadata <- front_matter
+
   # call any pre_knit handler
   if (!is.null(output_format$pre_knit)) {
     output_format$pre_knit(input = original_input)
@@ -557,6 +585,18 @@ render <- function(input,
     templates <- knitr::opts_template$get()
     on.exit(knitr::opts_template$restore(templates), add = TRUE)
 
+    # specify that htmltools::htmlPreserve should use the pandoc raw
+    # attribute (e.g. ```{=html}) rather than preservation tokens when
+    # pandoc >= v2.0. Note that this option will have the intended effect
+    # only for versions of htmltools >= 0.5.0.9003.
+    if (pandoc2.0() && packageVersion("htmltools") >= "0.5.0.9003") {
+      prev <- getOption("htmltools.preserve.raw", default = NA)
+      options(htmltools.preserve.raw = TRUE)
+      if (!is.na(prev)) {
+        on.exit(options(htmltools.preserve.raw = prev), add = TRUE)
+      }
+    }
+
     # run render on_exit (run after the knit hooks are saved so that
     # any hook restoration can take precedence)
     if (is.function(output_format$on_exit))
@@ -699,20 +739,6 @@ render <- function(input,
       }, add = TRUE)
     }
 
-    # make the front_matter available as 'metadata' within the knit environment
-    # (unless it is already defined there, in which case we emit a warning)
-    env <- environment(render)
-    metadata_this <- env$metadata
-    do.call("unlockBinding", list("metadata", env))
-    on.exit({
-      if (bindingIsLocked("metadata", env)) {
-        do.call("unlockBinding", list("metadata", env))
-      }
-      env$metadata <- metadata_this
-      lockBinding("metadata", env)
-    }, add = TRUE)
-    env$metadata <- front_matter
-
     # call onKnit hooks (normalize to list)
     sapply(as.list(getHook("rmarkdown.onKnit")), function(hook) {
       tryCatch(hook(input = original_input), error = function(e) NULL)
@@ -756,7 +782,7 @@ render <- function(input,
 
   # if this isn't html and there are html dependencies then flag an error
   if (!(is_pandoc_to_html(output_format$pandoc) ||
-        identical(tolower(tools::file_ext(output_file)), "html")))  {
+        identical(tolower(xfun::file_ext(output_file)), "html")))  {
     if (has_html_dependencies(knit_meta)) {
       if (!isTRUE(front_matter$always_allow_html)) {
         stop("Functions that produce HTML output found in document targeting ",
@@ -780,7 +806,7 @@ render <- function(input,
   # the knitr cache is active; clean the figure-* dir instead of the whole
   # files_dir if other subdirs are generated by another format and still needed:
   # https://github.com/rstudio/rmarkdown/issues/1472 and also #1503
-  if (output_format$clean_supporting && !dir_exists(cache_dir)) {
+  intermediates_fig <- if (output_format$clean_supporting && !dir_exists(cache_dir)) {
     # unlink does not support / at the end of file path
     fig_path <- gsub("/$", "", knitr::opts_chunk$get('fig.path'))
 
@@ -788,15 +814,13 @@ render <- function(input,
     # if no figure is generated, clean the whole files_dir (#1664)
     files_dir_fig <- list.files(files_dir, '^figure-.+')
 
-    intermediates <- c(
-      intermediates,
-      if (length(files_dir_fig) < 1 || identical(files_dir_fig, basename(fig_path))) {
-        files_dir
-      } else {
-        fig_path
-      }
-    )
+    if (length(files_dir_fig) < 1 || identical(files_dir_fig, basename(fig_path))) {
+      files_dir
+    } else {
+      fig_path
+    }
   }
+  intermediates <- c(intermediates, intermediates_fig)
 
   file.copy(input, utf8_input, overwrite = TRUE)
 
@@ -849,35 +873,61 @@ render <- function(input,
       utf8_input <- path.expand(utf8_input)
       output     <- path.expand(output)
 
+      pandoc_args <- output_format$pandoc$args
+
+      # if Lua filters are provided, add the command line switch
+      if (!is.null(lua_filters <- output_format$pandoc$lua_filters)) {
+        lua_filters <- pandoc_lua_filter_args(lua_filters)
+      }
+      pandoc_args <- c(lua_filters, pandoc_args)
+
+      # in case the output format turns on the --file-scope flag, run its
+      # file_scope function to split the input into multiple files
+      input_files <- utf8_input
+      if (!is.null(output_format$file_scope) &&
+          length(inputs <- output_format$file_scope(utf8_input)) > 1) {
+
+        # add the --file-scope option
+        pandoc_args <- c(pandoc_args, "--file-scope")
+
+        # write the split content into *.split.md files
+        input_files <- unlist(lapply(inputs, function(input) {
+          file <- file_with_meta_ext(input$name, "split", "md")
+          file <- file.path(dirname(utf8_input), file)
+          write_utf8(input$content, file)
+          file
+        }))
+
+        # cleanup the split files after render
+        on.exit(unlink(input_files), add = TRUE)
+      }
+
       # if we don't detect any invalid shell characters in the
       # target path, then just call pandoc directly
       if (!grepl(.shell_chars_regex, output) && !grepl(.shell_chars_regex, utf8_input)) {
         return(pandoc_convert(
-          utf8_input, pandoc_to, output_format$pandoc$from, output,
-          citeproc, output_format$pandoc$args, !quiet
+          input_files, pandoc_to, output_format$pandoc$from, output,
+          citeproc, pandoc_args, !quiet
         ))
       }
 
       # render to temporary file (preserve extension)
       # this also ensures we don't pass a file path with invalid
       # characters to our pandoc invocation
-      file_ext <- tools::file_ext(output)
-      ext <- if (nzchar(file_ext))
-        paste(".", file_ext, sep = "")
-      else
-        ""
+      ext <- xfun::file_ext(output)
+      if (ext != '') ext <- paste0('.', ext)
 
       # render to a path in the current working directory
       # (avoid passing invalid characters to shell)
-      pandoc_output_tmp <- basename(tempfile("pandoc", tmpdir = getwd(), fileext = ext))
+      pandoc_output_tmp <- basename(tempfile("pandoc", getwd(), ext))
 
       # clean up temporary file on exit
       on.exit(unlink(pandoc_output_tmp), add = TRUE)
 
       # call pandoc to render file
       status <- pandoc_convert(
-        utf8_input, pandoc_to, output_format$pandoc$from, pandoc_output_tmp,
-        citeproc, output_format$pandoc$args, !quiet
+        input_files, pandoc_to, output_format$pandoc$from, pandoc_output_tmp,
+        citeproc, pandoc_args, !quiet
       )
 
       # construct output path (when passed only a file name to '--output',
@@ -918,7 +968,10 @@ render <- function(input,
         latexmk(texfile, output_format$pandoc$latex_engine, '--biblatex' %in% output_format$pandoc$args)
         file.rename(file_with_ext(texfile, "pdf"), output_file)
         # clean up the tex file if necessary
-        if (!output_format$pandoc$keep_tex) on.exit(unlink(texfile), add = TRUE)
+        if (!output_format$pandoc$keep_tex) {
+          texfile <- normalize_path(texfile)
+          on.exit(unlink(texfile), add = TRUE)
+        }
       }
     } else {
       convert(output_file, run_citeproc)
@@ -968,9 +1021,15 @@ render <- function(input,
       attr(output_file, 'rmd_output_metadata') <- output_meta
     invisible(output_file)
   } else {
+    # make sure the markdown output and fig dir are not cleaned up
+    intermediates <- setdiff(intermediates, c(input, intermediates_fig))
     # did not run pandoc; returns the markdown output with attributes of the
     # knitr meta data and intermediate files
-    structure(input, knit_meta = knit_meta, intermediates = intermediates)
+    structure(input,
+              knit_meta = knit_meta,
+              files_dir = files_dir,
+              intermediates_dir = intermediates_fig,
+              intermediates = intermediates)
   }
 }
 
